@@ -1,8 +1,8 @@
-# $Header: /Library/VersionControl/CVS/Perl-Metrics-Simple/lib/Perl/Metrics/Simple/Analysis/File.pm,v 1.20 2010/02/07 20:59:07 matisse Exp $
-# $Revision: 1.20 $
+# $Header: /Library/VersionControl/CVS/Perl-Metrics-Simple/lib/Perl/Metrics/Simple/Analysis/File.pm,v 1.23 2010/05/09 18:02:28 matisse Exp $
+# $Revision: 1.23 $
 # $Author: matisse $
 # $Source: /Library/VersionControl/CVS/Perl-Metrics-Simple/lib/Perl/Metrics/Simple/Analysis/File.pm,v $
-# $Date: 2010/02/07 20:59:07 $
+# $Date: 2010/05/09 18:02:28 $
 ###############################################################################
 
 package Perl::Metrics::Simple::Analysis::File;
@@ -17,25 +17,67 @@ use PPI;
 use PPI::Document;
 use Readonly;
 
-our $VERSION = '0.14';
+our $VERSION = '0.15';
 
 Readonly::Scalar my $ALL_NEWLINES_REGEX =>
     qr/ ( \Q$INPUT_RECORD_SEPARATOR\E ) /sxm;
-Readonly::Array our @LOGIC_OPERATORS =>
-    qw( ! && || ||= &&= or and xor not ? <<= >>= );
-Readonly::Hash our %LOGIC_OPERATORS => hashify(@LOGIC_OPERATORS);
+Readonly::Array our @DEFAULT_LOGIC_OPERATORS => qw(
+    !
+    !~
+    &&
+    &&=
+    //
+    <
+    <<=
+    <=>
+    ==
+    =~
+    >
+    >>=
+    ?
+    and
+    cmp
+    eq
+    gt
+    lt
+    ne
+    not
+    or
+    xor
+    ||
+    ||=
+    ~~
+    );
 
-Readonly::Array our @LOGIC_KEYWORDS =>
-    qw( for foreach goto if else elsif last next unless until while );
-Readonly::Hash our %LOGIC_KEYWORDS => hashify(@LOGIC_KEYWORDS);
+Readonly::Array our @DEFAULT_LOGIC_KEYWORDS => qw(
+    else
+    elsif
+    for
+    foreach 
+    goto
+    grep
+    if
+    last
+    map
+    next
+    unless
+    until
+    while
+    );
 Readonly::Scalar my $LAST_CHARACTER => -1;
 
+our (@LOGIC_KEYWORDS, @LOGIC_OPERATORS); # For user-supplied values;
+
+our (%LOGIC_KEYWORDS, %LOGIC_OPERATORS); # Populated in _init()
+
 # Private instance variables:
-my %Path       = ();
-my %Main_Stats = ();
-my %Subs       = ();
-my %Packages   = ();
-my %Lines      = ();
+my %_PATH       = ();
+my %_MAIN_STATS = ();
+my %_SUBS       = ();
+my %_PACKAGES   = ();
+my %_LINES      = ();
+my %_LOGIC_KEYWORDS = ();
+my %_LOGIC_OPERATORS = ();
 
 sub new {
     my ( $class, %parameters ) = @_;
@@ -47,9 +89,9 @@ sub new {
 
 sub _init {
     my ( $self, %parameters ) = @_;
-    $Path{$self} = $parameters{path};
+    $_PATH{$self} = $parameters{'path'};
 
-    my $path = $self->path;
+    my $path = $self->path();
 
     my $document;
     if (ref $path) {
@@ -73,15 +115,23 @@ sub _init {
 
     my $packages = _get_packages($document);
 
+    my @logic_keywords = @LOGIC_KEYWORDS  ? @LOGIC_KEYWORDS : @DEFAULT_LOGIC_KEYWORDS;
+    %LOGIC_KEYWORDS = hashify(@logic_keywords);
+    $_LOGIC_OPERATORS{$self} = \%LOGIC_KEYWORDS;
+
+    my @logic_operators = @LOGIC_OPERATORS ? @LOGIC_OPERATORS : @DEFAULT_LOGIC_OPERATORS;
+    %LOGIC_OPERATORS = hashify(@logic_operators);
+    $_LOGIC_OPERATORS{$self} = \%LOGIC_OPERATORS;
+
     my @sub_analysis = ();
     my $sub_elements = $document->find('PPI::Statement::Sub');
     @sub_analysis = @{ $self->_iterate_over_subs($sub_elements) };
 
-    $Main_Stats{$self}
+    $_MAIN_STATS{$self}
         = $self->analyze_main( $document, $sub_elements, \@sub_analysis );
-    $Subs{$self}     = \@sub_analysis;
-    $Packages{$self} = $packages;
-    $Lines{$self}    = $self->get_node_length($document);
+    $_SUBS{$self}     = \@sub_analysis;
+    $_PACKAGES{$self} = $packages;
+    $_LINES{$self}    = $self->get_node_length($document);
 
     return $self;
 }
@@ -179,27 +229,37 @@ sub get_node_length {
 
 sub path {
     my ($self) = @_;
-    return $Path{$self};
+    return $_PATH{$self};
 }
 
 sub main_stats {
     my ($self) = @_;
-    return $Main_Stats{$self};
+    return $_MAIN_STATS{$self};
 }
 
 sub subs {
     my ($self) = @_;
-    return $Subs{$self};
+    return $_SUBS{$self};
 }
 
 sub packages {
     my ($self) = @_;
-    return $Packages{$self};
+    return $_PACKAGES{$self};
 }
 
 sub lines {
     my ($self) = @_;
-    return $Lines{$self};
+    return $_LINES{$self};
+}
+
+sub logic_keywords {
+    my ($self) = @_;
+    return wantarray ? @{$_LOGIC_KEYWORDS{$self}} : $_LOGIC_KEYWORDS{$self};
+}
+
+sub logic_operators {
+    my ($self) = @_;
+    return wantarray ? @{$_LOGIC_OPERATORS{$self}} : $_LOGIC_OPERATORS{$self};
 }
 
 sub measure_complexity {
@@ -401,6 +461,16 @@ Total non-blank, non-comment, non-pod lines.
 
 Returns the hashref generated by I<analyze_main> without re-analyzing document.
 
+=head2 logic_keywords
+
+Returns an array (in array context) or ref-to-ARRAY of the keywords
+used in calculating complexity. See I<Logic Keywords> section below.
+
+=head2 logic_operators
+
+Returns an array (in array context) or ref-to-ARRAY of the operators
+used in calculating complexity. See I<Logic Operators> section below.
+
 =head2 measure_complexity
 
 Takes a B<PPI> element and measures an approximation of the
@@ -417,15 +487,61 @@ The actual measurement we use for a chunk of code is 1 plus
 
 =head3 Logic operators:
 
-See I<@Perl::Metrics::Simple::Analysis::File::LOGIC_OPERATORS>
+The default list is:
 
- ! && || ||= &&= or and xor not ? <<= >>=
+I<@Perl::Metrics::Simple::Analysis::File::DEFAULT_LOGIC_OPERATORS>
+
+    !
+    !~
+    &&
+    &&=
+    //
+    <
+    <<=
+    <=>
+    ==
+    =~
+    >
+    >>=
+    ?
+    and
+    cmp
+    eq
+    gt
+    lt
+    ne
+    not
+    or
+    xor
+    ||
+    ||=
+    ~~
+
+You can supply your own list by setting:
+I<@Perl::Metrics::Simple::Analysis::File::LOGIC_OPERATORS> before creating a
+new object.
 
 =head3 Logic keywords:
 
-See I<@Perl::Metrics::Simple::Analysis::File::LOGIC_KEYWORDS>
+I<@Perl::Metrics::Simple::Analysis::File::DEFAULT_LOGIC_KEYWORDS>
 
- for foreach goto if else elsif last next unless until while
+    else
+    elsif
+    for
+    foreach 
+    goto
+    grep
+    if
+    last
+    map
+    next
+    unless
+    until
+    while
+
+You can supply your own list by setting:
+I<@Perl::Metrics::Simple::Analysis::File::LOGIC_KEYWORDS> before creating a
+new object.
 
 =head3 Examples of Complexity
 
